/***************************************************************************
 *   Copyright (C) 2006 by Martin Strasser                                 *
 *   strasser  a t  cadmaniac  d o t  org                                  *
 *   Special thanks to Mario Boikov                                        *
 *   squeeze  a t  cadmaniac  d o t  org                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "klavrgcccompiler.h"
#include "kldocument.h"
#include "klproject.h"
#include "klprocess.h"
#include "klprocessmanager.h"
#include "kontrollerlab.h"
#include <klocale.h>

KLAVRGCCCompiler::KLAVRGCCCompiler( KLProject* project, KLProcessManager* processManager )
 : KLCompilerInterface( project, processManager )
{
}


KLAVRGCCCompiler::~KLAVRGCCCompiler()
{
}

void KLAVRGCCCompiler::compile( KLDocument* what )
{
    if ( what->type() != KLDocType_Source )
        return;
    // m_cmdSchedule.append( "cd " + m_project->projectBaseURL().path() );
    m_processManager->setCWD( m_project->projectBaseURL().path() );
    // First, save all of them:
    if ( what->kateDoc()->isModified() )
        what->save();
    QFileInfo srcFI( what->url().path() );
    QFileInfo objFI( m_project->getObjectFileNameFor( what->url() ) );
    // Do we need to compile the file?
    if ( !( objFI.exists() && ( objFI.lastModified() > srcFI.lastModified() ) ) )
        m_cmdSchedule.append( getCompileCommandFor( what->url() ) );

    // m_cmdSchedule.append( getLinkerCommand().ascii() );
    // m_cmdSchedule.append( getObjCopyCommand() );
    QString firstCmd = m_cmdSchedule.front();
    m_cmdSchedule.pop_front();
    m_processManager->addProcess( firstCmd, (KLProcessHandler*) this );
}


void KLAVRGCCCompiler::buildAll( bool forceRebuild )
{
    KLDocument *it;
    // m_cmdSchedule.append( "cd " + m_project->projectBaseURL().path() );
    m_processManager->setCWD( m_project->projectBaseURL().path() );
    // First, save all of them:
    QPtrList< KLDocument > list = m_project->getAllDocsOfType( KLDocType_Source );
    for ( it = list.first(); it; it = list.next() )
    {
        // qDebug( "%s is modified: %d", it->name().ascii(), it->kateDoc()->isModified() );
        if ( it->kateDoc()->isModified() )
            it->save();
        QFileInfo srcFI( it->url().path() );
        QFileInfo objFI( m_project->getObjectFileNameFor( it->url() ) );
        // Do we need to compile the file?
        if ( ( !( objFI.exists() && ( objFI.lastModified() > srcFI.lastModified() ) ) ) || forceRebuild )
            // m_cmdSchedule.append( getCompileCommandFor( it->url() ) );
            m_processManager->addProcess( getCompileCommandFor( it->url() ), (KLProcessHandler*) this );
    }
    if ( list.count() > 0 )
    {
        m_processManager->addProcess( getLinkerCommand(), (KLProcessHandler*) this );
        m_processManager->addProcess( getObjCopyCommand(), (KLProcessHandler*) this );
        // m_cmdSchedule.append( getLinkerCommand() );
        // m_cmdSchedule.append( getObjCopyCommand() );
        // QString firstCmd = m_cmdSchedule.front();
        // m_cmdSchedule.pop_front();
        // m_processManager->addProcess( firstCmd, (KLProcessHandler*) this );
    }
}


QString KLAVRGCCCompiler::getCompileCommandFor( const KURL & file )
{
    QString retVal;

    // There is a difference if it is an assembler (.s) or a C file:
    if ( file.filename().lower().endsWith( ".s" ) )
    {
        retVal = m_project->m_settings[ PRJ_ASSEMBLER_COMMAND ];
        
        retVal += " -mmcu=" + m_project->currentCpuFeatures().mmcuFlag();
        retVal += m_project->attribute( "-g", PRJ_COMPILER_GDEBUG, true );
        retVal += m_project->attribute( "-O", PRJ_COMPILER_OPT_LEVEL );
        
        // Append the clock speed if desired:
        if ( m_project->attribute( "", PRJ_CLOCKSPEED ).stripWhiteSpace().length() > 0 )
        {
            bool ok;
            double val = m_project->attribute( "", PRJ_CLOCKSPEED ).stripWhiteSpace().toDouble( &ok );
            if (ok && (m_project->conf( PRJ_COMPILER_F_CPU, FALSE_STRING ) == TRUE_STRING) )
                retVal += QString(" -DF_CPU=%1UL").arg(val, 0, 'f', 0);
        }
        
        retVal += " -c " + file.fileName() + " -o " + m_project->getObjectFileNameFor( file );
    }
    else
    {
        retVal = m_project->m_settings[ PRJ_COMPILER_COMMAND ];
        
        retVal += " -mmcu=" + m_project->currentCpuFeatures().mmcuFlag();
        retVal += m_project->attribute( "-mcall-prologues", PRJ_COMPILER_CALL_PROLOGUES, true );
        retVal += m_project->attribute( "-Wstrict-prototypes", PRJ_COMPILER_STRICT_PROTOTYPES, true );
        retVal += m_project->attribute( "-Wall", PRJ_COMPILER_WALL, true );
        retVal += m_project->attribute( "-g", PRJ_COMPILER_GDEBUG, true );
        retVal += m_project->attribute( "-O", PRJ_COMPILER_OPT_LEVEL );
        if ( m_project->attribute( "", PRJ_CLOCKSPEED ).stripWhiteSpace().length() > 0 )
        {
            bool ok;
            double val = m_project->attribute( "", PRJ_CLOCKSPEED ).stripWhiteSpace().toDouble( &ok );
            if (ok && (m_project->conf( PRJ_COMPILER_F_CPU, FALSE_STRING ) == TRUE_STRING) )
                retVal += QString(" -DF_CPU=%1UL").arg(val, 0, 'f', 0);
        }
        
        retVal += " -c " + file.fileName() + " -o " + m_project->getObjectFileNameFor( file );
    }
    return retVal;
}


QString KLAVRGCCCompiler::getLinkerCommand( )
{
    QString retVal = m_project->m_settings[ PRJ_LINKER_COMMAND ];
    
    retVal += " -mmcu=" + m_project->currentCpuFeatures().mmcuFlag();
    retVal += m_project->attribute( "-g", PRJ_COMPILER_GDEBUG, true );
    retVal += m_project->attributeLinkerSection( "-Wl,-Ttext=0x",
                         PRJ_LINKER_START_TEXT );
    retVal += m_project->attributeLinkerSection( "-Wl,-Tdata=0x",
                         PRJ_LINKER_START_DATA );
    retVal += m_project->attributeLinkerSection( "-Wl,-Tbss=0x",
                         PRJ_LINKER_START_BSS );
    retVal += m_project->attributeLinkerSection( "-Wl,--defsym=__heap_start=0x",
                         PRJ_LINKER_START_HEAP );
    retVal += m_project->attributeLinkerSection( "-Wl,--defsym=__heap_end=0x",
                         PRJ_LINKER_END_HEAP );
    
    retVal += " -o " + m_project->getOUTFileName();
    retVal += " -Wl,-Map," + m_project->getMAPFileName();
    
    QString buf = m_project->conf( PRJ_LINKER_FLAGS, "" ).stripWhiteSpace();
    buf.replace( "#", "," );
    if ((!buf.isEmpty()) && (!buf.isNull()))
        retVal += " " + buf;
    
    KLDocument *it;
    // First, save all of them:
    QPtrList< KLDocument > list = m_project->getAllDocsOfType( KLDocType_Source );
    for ( it = list.first(); it; it = list.next() )
    {
        retVal += " " + m_project->getObjectFileNameFor( it->url() );
    }
    
    return retVal;
}


QString KLAVRGCCCompiler::getObjCopyCommand( )
{
    QString retVal = m_project->m_settings[ PRJ_OBJCOPY_COMMAND ];
    retVal += " -R .eeprom -O ihex " + m_project->getOUTFileName() +
            " " + m_project->getHEXFileName();
    return retVal;
}

void KLAVRGCCCompiler::shellProcessExitedWith( int exitVal, const QString & stdout, const QString & stderr )
{
    if ( (!stdout.isEmpty()) && (!stdout.isNull()) )
        m_project->parent()->slotMessageBox( exitVal, stdout );
    // qDebug("Stdout: %s", stdout.ascii() );
    if ( (!stderr.isEmpty()) && (!stderr.isNull()) )
        m_project->parent()->slotMessageBox( exitVal, stderr );
    // qDebug("Stderr: %s", stderr.ascii() );
    
    if (exitVal == 0)
    {
        if (m_cmdSchedule.count() > 0 )
        {
            QString nextCmd = m_cmdSchedule.front();
            m_cmdSchedule.pop_front();
            m_processManager->setCWD( m_project->projectBaseURL().path() );
            m_processManager->addProcess( nextCmd, (KLProcessHandler*)this );
        }
        else
            m_project->updateHexFileSize();
    }
    else
    {
        m_project->parent()->slotMessageBox( exitVal, i18n("Error(s) occurred: The exit status was %1.").arg( exitVal ) );
        // qDebug("There were problems. Exit was %d.", exitVal);
        m_cmdSchedule.clear();
    }
}
