/***************************************************************************
 *   Copyright (C) 2006 by Martin Strasser                                 *
 *   strasser  a t  cadmaniac  d o t  org                                  *
 *   Special thanks to Mario Boikov                                        *
 *   squeeze  a t  cadmaniac  d o t  org                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "klfilenewdialog.h"
#include "kontrollerlab.h"
#include "klproject.h"
#include "kldocument.h"
#include "kldocumentview.h"
#include <qlineedit.h>
#include <kurl.h>
#include <qiconview.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kglobal.h>


KLFileNewDialog::KLFileNewDialog(KontrollerLab *parent, KLProject* project, const char *name)
    :KLFileNewDialogBase(parent, name)
{
    m_parent = parent;
    m_project = project;
    
    KIconLoader *iconLoader = KGlobal::iconLoader();
    m_cSource = new QIconViewItem( klNewFile, i18n("C source"), iconLoader->loadIcon( "source_c", KIcon::Desktop ) );
    m_asmSource = new QIconViewItem( klNewFile, i18n("ASM source"), iconLoader->loadIcon( "source_s", KIcon::Desktop ) );
    m_cHeader = new QIconViewItem( klNewFile, i18n("C header"), iconLoader->loadIcon( "source_h", KIcon::Desktop ) );
    m_txt = new QIconViewItem( klNewFile, i18n("Text"), iconLoader->loadIcon( "txt", KIcon::Desktop ) );
    m_other = new QIconViewItem( klNewFile, i18n("Other"), iconLoader->loadIcon( "unknown", KIcon::Desktop ) );
}

void KLFileNewDialog::slotCancel()
{
    close();
}


void KLFileNewDialog::slotOK()
{
    if (leName->text().isEmpty() || leName->text().isNull())
        return;

    KLDocument* doc = new KLDocument(m_parent);
    KURL newFileURL = m_project->projectBaseURL();
    QString fname = leName->text();
    
    if ( klNewFile->currentItem() == m_cSource )
    {
        fname = fname.lower().endsWith(".c") ? fname : fname + ".c";
    }
    else if ( klNewFile->currentItem() == m_asmSource )
    {
        fname = fname.lower().endsWith(".s") ? fname : fname + ".s";
    }
    else if ( klNewFile->currentItem() == m_cHeader )
    {
        fname = fname.lower().endsWith(".h") ? fname : fname + ".h";
    }
    else if ( klNewFile->currentItem() == m_txt )
    {
        fname = fname.lower().endsWith(".txt") ? fname : fname + ".txt";
    }

    newFileURL.addPath( fname );

    if ( !newFileURL.isValid() )
        return;
    doc->newFile( newFileURL );
    doc->kateDoc()->setModified( true );
    m_project->addDocument( doc );

    m_parent->slotNewPart( doc->kateDoc(), true );
    KLDocumentView *view = new KLDocumentView( doc, m_parent );

    close();
}

void KLFileNewDialog::slotFileNameChanged( const QString& fn )
{
    pbOK->setEnabled( fn.length() > 0 ? true: false );
}



#include "klfilenewdialog.moc"
