/***************************************************************************
 *   Copyright (C) 2006 by Martin Strasser                                 *
 *   strasser  a t  cadmaniac  d o t  org                                  *
 *   Special thanks to Mario Boikov                                        *
 *   squeeze  a t  cadmaniac  d o t  org                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "klprogrammeravrdude.h"
#include "klproject.h"


KLProgrammerAVRDUDE::KLProgrammerAVRDUDE(KLProcessManager* processManager, KLProject* project)
 : KLProgrammerInterface(processManager, project)
{
    m_mmcuLookup[QString("ATtiny84").upper()] = "t84";
    m_mmcuLookup[QString("ATtiny44").upper()] = "t44";
    m_mmcuLookup[QString("ATtiny24").upper()] = "t24";
    m_mmcuLookup[QString("ATMEGA1281").upper()] = "m1281";
    m_mmcuLookup[QString("ATMEGA1280").upper()] = "m1280";
    m_mmcuLookup[QString("ATMEGA640").upper()] = "m640";
    m_mmcuLookup[QString("ATtiny85").upper()] = "t85";
    m_mmcuLookup[QString("ATtiny45").upper()] = "t45";
    m_mmcuLookup[QString("ATtiny25").upper()] = "t25";
    m_mmcuLookup[QString("AT90PWM3").upper()] = "pwm3";
    m_mmcuLookup[QString("AT90PWM2").upper()] = "pwm2";
    m_mmcuLookup[QString("ATtiny2313").upper()] = "t2313";
    m_mmcuLookup[QString("ATMEGA168").upper()] = "m168";
    m_mmcuLookup[QString("ATMEGA88").upper()] = "m88";
    m_mmcuLookup[QString("ATMEGA48").upper()] = "m48";
    m_mmcuLookup[QString("ATTINY26").upper()] = "t26";
    m_mmcuLookup[QString("ATMEGA8535").upper()] = "m8535";
    m_mmcuLookup[QString("ATMEGA8515").upper()] = "m8515";
    m_mmcuLookup[QString("ATMEGA8").upper()] = "m8";
    m_mmcuLookup[QString("ATMEGA161").upper()] = "m161";
    m_mmcuLookup[QString("ATMEGA32").upper()] = "m32";
    m_mmcuLookup[QString("ATMEGA6490").upper()] = "m6490";
    m_mmcuLookup[QString("ATMEGA649").upper()] = "m649";
    m_mmcuLookup[QString("ATMEGA3290").upper()] = "m3290";
    m_mmcuLookup[QString("ATMEGA329").upper()] = "m329";
    m_mmcuLookup[QString("ATMEGA169").upper()] = "m169";
    m_mmcuLookup[QString("ATMEGA163").upper()] = "m163";
    m_mmcuLookup[QString("ATMEGA162").upper()] = "m162";
    m_mmcuLookup[QString("ATMEGA644").upper()] = "m644";
    m_mmcuLookup[QString("ATMEGA324").upper()] = "m324";
    m_mmcuLookup[QString("ATMEGA164").upper()] = "m164";
    m_mmcuLookup[QString("ATMEGA16").upper()] = "m16";
    m_mmcuLookup[QString("AT90CAN128").upper()] = "c128";
    m_mmcuLookup[QString("ATMEGA128").upper()] = "m128";
    m_mmcuLookup[QString("ATMEGA64").upper()] = "m64";
    m_mmcuLookup[QString("ATMEGA103").upper()] = "m103";
    m_mmcuLookup[QString("AT90S8535").upper()] = "8535";
    m_mmcuLookup[QString("AT90S8515").upper()] = "8515";
    m_mmcuLookup[QString("AT90S4434").upper()] = "4434";
    m_mmcuLookup[QString("AT90S4433").upper()] = "4433";
    m_mmcuLookup[QString("AT90S2343").upper()] = "2343";
    m_mmcuLookup[QString("AT90S2333").upper()] = "2333";
    m_mmcuLookup[QString("AT90S2313").upper()] = "2313";
    m_mmcuLookup[QString("AT90S4414").upper()] = "4414";
    m_mmcuLookup[QString("AT90S1200").upper()] = "1200";
    m_mmcuLookup[QString("ATtiny15").upper()] = "t15";
    m_mmcuLookup[QString("ATtiny13").upper()] = "t13";
    m_mmcuLookup[QString("ATtiny12").upper()] = "t12";
}


KLProgrammerAVRDUDE::~KLProgrammerAVRDUDE()
{
}


bool KLProgrammerAVRDUDE::erase( )
{
    m_runningProc = true;
    m_processManager->addProcess( formBaseCommand() + " -e", this );
    return true;
}


bool KLProgrammerAVRDUDE::upload( const KURL & url )
{
    m_runningProc = true;
    m_processManager->addProcess( formBaseCommand() + " -U flash:w:" + validPath(url) + ":i", this );
    return true;
}


bool KLProgrammerAVRDUDE::verify( const KURL & url )
{
    m_runningProc = true;
    m_processManager->addProcess( formBaseCommand() + " -U flash:v:" + validPath(url) + ":i", this );
    return true;
}


bool KLProgrammerAVRDUDE::download( const KURL & url )
{
    m_runningProc = true;
    m_processManager->addProcess( formBaseCommand() + " -U flash:r:" + validPath(url) + ":i", this );
    return true;
}


bool KLProgrammerAVRDUDE::ignite( const KURL & url )
{
    m_runningProc = true;
    m_processManager->addProcess( formBaseCommand() + " -U flash:w:" + validPath(url) + ":i", this );
    return true;
}


bool KLProgrammerAVRDUDE::programFuses( QMap< QString, QString > fuses )
{
    m_runningProc = true;
    QString cmd = formBaseCommand();
    m_readFusesBackannotate = 0;
    m_processManager->setDontAbortQueueOnError(true);
    if ( fuses.contains( FUSE_L ) )
    {
        m_readFusesBackannotate = 2;
        m_processManager->addProcess( formBaseCommand() + " -U lfuse:w:" + fuses[ FUSE_L ] + ":m", this );
        m_processManager->addProcess( formBaseCommand() + " -U fuse:w:" + fuses[ FUSE_L ] + ":m", this );
    }
    if ( fuses.contains( FUSE_H ) )
    {
        m_readFusesBackannotate++;
        m_processManager->addProcess( formBaseCommand() + " -U hfuse:w:" + fuses[ FUSE_H ] + ":m", this );
    }
    if ( fuses.contains( FUSE_E ) )
    {
        m_readFusesBackannotate++;
        m_processManager->addProcess( formBaseCommand() + " -U efuse:w:" + fuses[ FUSE_E ] + ":m", this );;
    }
    if ( fuses.contains( FUSE_LOCK ) )
    {
        m_readFusesBackannotate++;
        m_processManager->addProcess( formBaseCommand() + " -U lock:w:" + fuses[ FUSE_LOCK ] + ":m", this );
    }

    return true;
}


QString KLProgrammerAVRDUDE::attribute( const QString & attr, const QString confKey ) const
{
    QString retVal;
    if ( m_configuration.contains( confKey ) )
    {
        retVal = m_configuration[ confKey ];
        if ( (!retVal.isNull()) && (!retVal.isEmpty()) )
            return " " + attr + retVal;
    }
    return "";
}


QString KLProgrammerAVRDUDE::formBaseCommand( ) const
{
    QString cmd = m_configuration[ PROGRAMMER_COMMAND ] + " ";
    QString mmcu = m_mmcuLookup[m_project->currentCpuFeatures().name().upper()];
    if ( (!mmcu.isEmpty()) && (!mmcu.isNull()) )
        cmd += " -p " + mmcu;
    cmd += attribute( "-b ", AVRDUDE_OVERRIDE_BAUD_RATE );
    cmd += attribute( "-B ", AVRDUDE_SPECIFY_BIT_CLOCK );
    cmd += attribute( "-c ", AVRDUDE_PROGRAMMER_TYPE );
    cmd += attribute( "-P ", AVRDUDE_CONNECTION_PORT );
    cmd += attribute( "-C ", AVRDUDE_EXTERNAL_CONFIG_FILE );
    if ( m_configuration[ AVRDUDE_DISABLE_AUTO_ERASE ] == TRUE_STRING )
        cmd += " -D";
    if ( m_configuration[ AVRDUDE_TEST_MODE ] == TRUE_STRING )
        cmd += " -n";
    if ( m_configuration[ AVRDUDE_OVERRIDE_INVALID_SIGNATURE ] == TRUE_STRING )
        cmd += " -F";
    if ( m_configuration[ AVRDUDE_COUNT_ERASE ] == TRUE_STRING )
        cmd += " -y";
    // Disable safemode:
    return cmd + " -u ";
}


void KLProgrammerAVRDUDE::setEraseCounter( int cnt )
{
    m_runningProc = true;
    m_processManager->addProcess( formBaseCommand() + QString(" -Y %1").arg(cnt), this );
}


bool KLProgrammerAVRDUDE::readFuses( )
{
    m_runningProc = true;
    m_readFusesBackannotate = true;
    m_processManager->setCWD( m_project->projectBaseURL() );
    KURL fuseURL( m_project->projectBaseURL() );
    fuseURL.addPath( ".fuse" );
    KURL lfuseURL( m_project->projectBaseURL() );
    lfuseURL.addPath( ".lfuse" );
    KURL hfuseURL( m_project->projectBaseURL() );
    hfuseURL.addPath( ".hfuse" );
    KURL efuseURL( m_project->projectBaseURL() );
    efuseURL.addPath( ".efuse" );
    KURL lockURL( m_project->projectBaseURL() );
    lockURL.addPath( ".lock" );
    // Remove all old fuse files:
    QFile fuseFile( fuseURL.path() );
    fuseFile.remove();
    QFile lfuseFile( lfuseURL.path() );
    lfuseFile.remove();
    QFile hfuseFile( hfuseURL.path() );
    hfuseFile.remove();
    QFile efuseFile( efuseURL.path() );
    efuseFile.remove();
    QFile lockFile( lockURL.path() );
    lockFile.remove();
    // Now let them be read:
    m_readFusesBackannotate = 5;
    m_processManager->setDontAbortQueueOnError( true );
    m_processManager->addProcess( formBaseCommand() + " -U fuse:r:.fuse:r", this );
    m_processManager->addProcess( formBaseCommand() + " -U lfuse:r:.lfuse:r", this );
    m_processManager->addProcess( formBaseCommand() + " -U hfuse:r:.hfuse:r", this );
    m_processManager->addProcess( formBaseCommand() + " -U efuse:r:.efuse:r", this );
    m_processManager->addProcess( formBaseCommand() + " -U lock:r:.lock:r", this );

    return true;
}


QMap< QString, QString > KLProgrammerAVRDUDE::parseFuses( const QString & )
{
    QMap< QString, QString > retVal;
    
    KURL fuseURL( m_project->projectBaseURL() );
    fuseURL.addPath( ".fuse" );
    KURL lfuseURL( m_project->projectBaseURL() );
    lfuseURL.addPath( ".lfuse" );
    KURL hfuseURL( m_project->projectBaseURL() );
    hfuseURL.addPath( ".hfuse" );
    KURL efuseURL( m_project->projectBaseURL() );
    efuseURL.addPath( ".efuse" );
    KURL lockURL( m_project->projectBaseURL() );
    lockURL.addPath( ".lock" );
    QFile fuseFile( fuseURL.path() );
    QFile lfuseFile( lfuseURL.path() );
    QFile hfuseFile( hfuseURL.path() );
    QFile efuseFile( efuseURL.path() );
    QFile lockFile( lockURL.path() );
    
    readFromFile( fuseFile, FUSE_L, retVal );
    readFromFile( lfuseFile, FUSE_L, retVal );
    readFromFile( hfuseFile, FUSE_H, retVal );
    readFromFile( efuseFile, FUSE_E, retVal );
    readFromFile( lockFile, FUSE_LOCK, retVal );

    return retVal;
}


void KLProgrammerAVRDUDE::readFromFile( QFile & file, const QString & key, QMap< QString, QString > & map )
{
    QFileInfo finfo( file );

    // qDebug( "fsize is %d", finfo.size());
    if ( file.exists() && (finfo.size()==1) )
    {
        // qDebug("EXISTS: %s", file.name().ascii());
        if (file.open( IO_ReadOnly ))
        {
            int ch = file.getch();
            if ( ch >= 0 )
                map[ key ] = QString("0x%1").arg( ch, 0, 16 );
            // qDebug("%s is %s", key.ascii(), map[key].ascii());
            file.close();
        }
    }
}


