/***************************************************************************
 *   Copyright (C) 2006 by Martin Strasser                                 *
 *   strasser  a t  cadmaniac  d o t  org                                  *
 *   Special thanks to Mario Boikov                                        *
 *   squeeze  a t  cadmaniac  d o t  org                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "klprojectmanagerwidget.h"
#include "klproject.h"

#include <kiconloader.h>
#include <kglobal.h>
#include <klocale.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <kmessagebox.h>
#include "kontrollerlab.h"


KLProjectManagerWidget::KLProjectManagerWidget(KLProject* project, QWidget *parent, const char *name)
    :KLProjectManagerWidgetBase(parent, name)
{
    m_project = project;
    m_project->registerProjectManager( this );
    m_rootNode = m_sourcesNode = m_headersNode = m_notesNode = m_othersNode = 0L;
    m_iconLoader = KGlobal::iconLoader();
    m_curHexSize = 0;
    setProjectName( project->name() );
    m_projectManagerPopup = new KPopupMenu( this, "projectManagerPopup" );
    KIconLoader kico;
    m_projectManagerPopup->insertItem( kico.loadIcon( "edittrash", KIcon::Toolbar ), i18n("Remove from project"),
                                       this, SLOT( slotTrash() ) );
    
    connect( lvFiles, SIGNAL( rightButtonPressed( QListViewItem*, const QPoint& , int ) ),
             this, SLOT(slotShowPopupMenu( QListViewItem*, const QPoint& , int )  ) );

}

void KLProjectManagerWidget::slotDoubleClicked( QListViewItem* item )
{
    if ( !item )
        return;
    if ( item->rtti() == KLQListViewDocumentItem_RTTI )
    {
        KLQListViewDocumentItem *itemCast = (KLQListViewDocumentItem*) item;
        itemCast->doc()->makeLastActiveViewVisible();
        // qDebug( "%s", itemCast->doc()->name().ascii() );
    }
}

KLProjectManagerWidget::~ KLProjectManagerWidget( )
{
    m_project->unregisterProjectManager( this );
}

void KLProjectManagerWidget::addDocument( KLDocument * doc )
{
    KLQListViewDocumentItem *item;
    if ( doc->type() == KLDocType_Source )
    {
        item = new KLQListViewDocumentItem( m_sourcesNode, doc->name() );
        item->setPixmap( 0, m_iconLoader->loadIcon( "source_c", KIcon::Small ) );
    }
    else if ( doc->type() == KLDocType_Header )
    {
        item = new KLQListViewDocumentItem( m_headersNode, doc->name() );
        item->setPixmap( 0, m_iconLoader->loadIcon( "source_h", KIcon::Small ) );
    }
    else if ( doc->type() == KLDocType_Note )
    {
        item = new KLQListViewDocumentItem( m_notesNode, doc->name() );
        item->setPixmap( 0, m_iconLoader->loadIcon( "txt", KIcon::Small ) );
    }
    else if ( doc->type() == KLDocType_NoType )
    {
        item = new KLQListViewDocumentItem( m_othersNode, doc->name() );
        item->setPixmap( 0, m_iconLoader->loadIcon( "unknown", KIcon::Small ) );
    }
    item->setDoc( doc );
    item->updateFileInfo();
    // doc->setListViewItem( item );
}

void KLProjectManagerWidget::removeDocument( KLDocument * )
{
    update();
}

void KLProjectManagerWidget::update( )
{
    KLDocument *it;
    KLQListViewDocumentItem *item;
    QPtrList< KLDocument > documents = m_project->documents();

    clear();
    for ( it = documents.first(); it; it = documents.next() )
    {
        if ( it->type() == KLDocType_Source )
        {
            item = new KLQListViewDocumentItem( m_sourcesNode, it->name() );
        }
        else if ( it->type() == KLDocType_Header )
        {
            item = new KLQListViewDocumentItem( m_headersNode, it->name() );
        }
        else if ( it->type() == KLDocType_Note )
        {
            item = new KLQListViewDocumentItem( m_notesNode, it->name() );
        }
        else if ( it->type() == KLDocType_NoType )
        {
            item = new KLQListViewDocumentItem( m_othersNode, it->name() );
        }
        // it->setListViewItem( item );
        item->setDoc( it );
        item->updateFileInfo();
    }
    m_rootNode->setOpen( true );
}


void KLQListViewDocumentItem::updateFileInfo( )
{
    QString buf;
    if (m_doc)
    {
        if ( QFile::exists( m_doc->url().path() ) )
        {
            QFileInfo info( m_doc->url().path() );

            setText( 1, buf.setNum( info.size() ) );
            setText( 2, info.lastModified().toString() );
        }
    }
}

void KLProjectManagerWidget::update( KLDocument * doc )
{
    QListViewItem *item = 0, *outerItem = m_rootNode->firstChild();

    while ( outerItem )
    {
        item = outerItem->firstChild();
        while( item )
        {
            if ( (item->rtti() == KLQListViewDocumentItem_RTTI) &&
                  (((KLQListViewDocumentItem*)item)->doc() == doc) )
                break;
            item = item->nextSibling();
        }
        if ( item )
            break;
        outerItem = outerItem->nextSibling();
    }

    if ( item && doc )
    {
        item->setText( 0, doc->name() );
        ((KLQListViewDocumentItem*)item)->updateFileInfo();
    }
}

KLQListViewDocumentItem::KLQListViewDocumentItem( QListViewItem * parent, const QString & name )
    : QListViewItem( parent, name )
{
    m_doc = 0L;
}

void KLProjectManagerWidget::slotConfigure()
{
    m_project->parent()->slotConfigProject();
}


void KLProjectManagerWidget::slotTrash()
{
    QListViewItem *item = lvFiles->selectedItem();
    if ( !item )
        return;
    if (item->rtti() == KLQListViewDocumentItem_RTTI)
    {
        KLQListViewDocumentItem *castItem = (KLQListViewDocumentItem*) item;
        int retVal = KMessageBox::questionYesNo( this, i18n( "Do you really want "
                "to remove %1 from the project?" ).arg( castItem->doc()->name() ) );
        int saveRetVal = KMessageBox::No;
        if ( retVal == KMessageBox::Yes )
        {
            if ( castItem->doc()->kateDoc()->isModified() )
            {
                saveRetVal = KMessageBox::questionYesNoCancel( this, i18n( "The file %1 has been modified.\n"
                        "Shall it be saved now?").arg( castItem->doc()->name() ) );
                if ( saveRetVal == KMessageBox::Yes )
                    castItem->doc()->save();
                else if ( retVal == KMessageBox::Cancel )
                    return;
            }
            // In case the file was not saved, the user maybe wants to delete it from disk:
            if ( KMessageBox::No == saveRetVal )
            {
                retVal = KMessageBox::questionYesNoCancel( this, i18n( "Do you want "
                        "to remove %1 from disk too?" ).arg( castItem->doc()->name() ) );
                if ( retVal == KMessageBox::Yes )
                {
                    QFile file( castItem->doc()->url().path() );
                    file.remove();
                }
                else if ( retVal == KMessageBox::Cancel )
                    return;
            }
            m_project->removeDocument( castItem->doc() );
        }
        // m_project->update();
    }
}


void KLProjectManagerWidget::updateModified()
{
    QListViewItem *item = 0, *outerItem = m_rootNode->firstChild();

    while ( outerItem )
    {
        item = outerItem->firstChild();
        while( item )
        {
            if ( (item->rtti() == KLQListViewDocumentItem_RTTI) )
                ((KLQListViewDocumentItem*)item)->updateModified();

            item = item->nextSibling();
        }
        outerItem = outerItem->nextSibling();
    }
}

void KLQListViewDocumentItem::updateModified( )
{
    if ( m_doc->kateDoc()->isModified() )
        setText( 0, m_doc->name() + " *" );
    else
        setText( 0, m_doc->name() );
}

void KLProjectManagerWidget::slotShowPopupMenu( QListViewItem * item, const QPoint & pt, int )
{
    if ( !item )
        return;
    if ( item->rtti() == KLQListViewDocumentItem_RTTI )
        m_projectManagerPopup->popup( pt );
}



void KLProjectManagerWidget::clear( )
{
    lvFiles->clear();
    m_rootNode = new QListViewItem( lvFiles, m_project->name() );
    m_sourcesNode = new QListViewItem( m_rootNode, i18n("Sources") );
    m_headersNode = new QListViewItem( m_rootNode, i18n("Headers") );
    m_notesNode = new QListViewItem( m_rootNode, i18n("Notes") );
    m_othersNode = new QListViewItem( m_rootNode, i18n("Other") );
    m_rootNode->setOpen( true );
}


#include "klprojectmanagerwidget.moc"
