/***************************************************************************
 *   Copyright (C) 2006 by Martin Strasser                                 *
 *   strasser  a t  cadmaniac  d o t  org                                  *
 *   Special thanks to Mario Boikov                                        *
 *   squeeze  a t  cadmaniac  d o t  org                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "klserialterminalwidget.h"
#include <qtextedit.h>
#include <qcombobox.h>
#include <qtoolbutton.h>
#include <kiconloader.h>
#include "kontrollerlab.h"
#include <klocale.h>
#include "klserialportlistener.h"
#include <qregexp.h>
#include <qradiobutton.h>


KLSerialTerminalWidget::KLSerialTerminalWidget(KontrollerLab *parent, const char *name)
    :KLSerialTerminalWidgetBase(parent, name)
{
    m_parent = parent;
    m_serialPort = new KLSerialPort;
    QStringList baudRates;
    QString curBRate;
    
    curBRate = "110";
    m_baudConsts[ curBRate ] = B110;
    baudRates.append( curBRate );

    curBRate = "300";
    m_baudConsts[ curBRate ] = B300;
    baudRates.append( curBRate );

    curBRate = "600";
    m_baudConsts[ curBRate ] = B600;
    baudRates.append( curBRate );

    curBRate = "1200";
    m_baudConsts[ curBRate ] = B1200;
    baudRates.append( curBRate );

    curBRate = "2400";
    m_baudConsts[ curBRate ] = B2400;
    baudRates.append( curBRate );

    curBRate = "4800";
    m_baudConsts[ curBRate ] = B4800;
    baudRates.append( curBRate );

    curBRate = "9600";
    m_baudConsts[ curBRate ] = B9600;
    baudRates.append( curBRate );

    curBRate = "19200";
    m_baudConsts[ curBRate ] = B19200;
    baudRates.append( curBRate );

    curBRate = "38400";
    m_baudConsts[ curBRate ] = B38400;
    baudRates.append( curBRate );

    curBRate = "57600";
    m_baudConsts[ curBRate ] = B57600;
    baudRates.append( curBRate );

    curBRate = "115200";
    m_baudConsts[ curBRate ] = B115200;
    baudRates.append( curBRate );

    cbBaud->insertStringList( baudRates );
    
    QStringList ports;
    ports.append( "/dev/ttyS0" );
    ports.append( "/dev/ttyS1" );
    ports.append( "/dev/ttyS2" );
    ports.append( "/dev/ttyS3" );
    ports.append( "/dev/ttyUSB0" );

    cbDevice->insertStringList( ports );
    
    KIconLoader ico;
    tbOpenClose->setIconSet( ico.loadIcon( "connect_no", KIcon::Panel ) );
    
    tbClear->setIconSet( ico.loadIcon( "clear_left", KIcon::Panel ) );
    
    m_listener = 0L;
    m_displayIsHEX = false;

}


void KLSerialTerminalWidget::slotSend()
{
    QRegExp rx("[^#]#([0-9a-fA-F]?[0-9a-fA-F])");
    QString outData = teSendData->text();
    // Add a space so that [^#]# matches also at the beginning. Strip that " " afterwards...
    outData = " " + outData;

    QStringList list;
    int pos = 0;
    while ( pos >= 0 ) {
        pos = rx.search( outData, pos );
        if ( pos > -1 ) {
            list += rx.cap( 1 );
            pos  += rx.matchedLength();
        }
    }

    bool ok;
    for ( QStringList::iterator it = list.begin(); it != list.end(); ++it )
    {
        QRegExp rx2("([^#])#0*"+(*it));
        outData.replace( rx2, QString("\\1") + QChar(char(QString(*it).toInt(&ok,16))) );
    }

    outData.replace( "##", "#" );
    // Strip the white space added in the beginning:
    outData = outData.mid( 1, outData.length() );

    m_serialPort->write( outData );
}

void KLSerialTerminalWidget::slotShowAsHEX(bool val)
{
    if (!val)
        return;
    if ( !m_displayIsHEX )
    {
        teReceiveData->setText( asciiToHEX(m_receiveBuffer) );
        m_displayIsHEX = true;
    }
}


void KLSerialTerminalWidget::slotShowAsAscii(bool val)
{
    if (!val)
        return;
    if ( m_displayIsHEX )
    {
        teReceiveData->setText( m_receiveBuffer );
        m_displayIsHEX = false;
    }
}


void KLSerialTerminalWidget::slotReceived( const QString& what_ )
{
//     teReceiveData->setCursorPosition( teReceiveData->paragraphs(),
//                                       teReceiveData->length() );
    QString what = what_;
    m_receiveBuffer += what;
    if ( rbHEX->isChecked() )
        what = asciiToHEX (what);
    if ( teReceiveData->text().endsWith("\n") )
    {
        teReceiveData->append( what );
    }
    else
    {
        if ( rbHEX->isChecked() )
            teReceiveData->setText( asciiToHEX( m_receiveBuffer ) );
        else
            teReceiveData->setText( m_receiveBuffer );
    }
}

void KLSerialTerminalWidget::slotOpenClose()
{
    if ( !m_serialPort->isOpen() )
    {
        if ( !m_serialPort->open( cbDevice->currentText(), m_baudConsts[ cbBaud->currentText() ] ) )
        {
            m_parent->slotMessageBox( 1, i18n("Could not open port %1.").arg(cbDevice->currentText()) );
        }
        else if ( m_listener == 0L )
        {
            m_listener = new KLSerialPortListener( this, m_serialPort, "serialPortListener" );
            m_listener->setReceiverActive(true);
        }
    }
    else
    {
        m_serialPort->close();
    }
    KIconLoader ico;
    // Check again if it is open now:
    if ( m_serialPort->isOpen() )
    {
        tbOpenClose->setIconSet( ico.loadIcon( "connect_established", KIcon::Panel ) );
        cbDevice->setEnabled(false);
        cbBaud->setEnabled(false);
        pbSend->setEnabled(true);
    }
    else
    {
        tbOpenClose->setIconSet( ico.loadIcon( "connect_no", KIcon::Panel ) );
        cbDevice->setEnabled(true);
        cbBaud->setEnabled(true);
        pbSend->setEnabled(false);
    }
}


void KLSerialTerminalWidget::paintEvent( QPaintEvent* )
{
    if (m_listener)
        slotReceived( m_listener->fetchOutData() );
}

QString KLSerialTerminalWidget::asciiToHEX( const QString & inp )
{
    QString retVal = "";
    if (inp.length()==0)
        return "";
    char buffer[inp.length()+1];
    strcpy( buffer, inp.ascii() );
    for (unsigned int i=0; i<inp.length(); i++)
    {
        char out = buffer[i];
        if (out < 0)
            out+=128;
        retVal += QString("%1 ").arg( out, 2, 16 );
    }
    return retVal;
}

void KLSerialTerminalWidget::slotClearReceiver()
{
    m_receiveBuffer = "";
    teReceiveData->setText( m_receiveBuffer );
}





#include "klserialterminalwidget.moc"
